{% load i18n thumbnail %}


{% if original_image %}
    <a href="{{ original_image.url }}" target="_blank">
        <img src="{% thumbnail original_image "240x240" %}"/>
    </a>
{% else %}
    <p>{% trans "No image" %}</p>
{% endif %}
