{% load i18n %}

{% if episode %}
    <div class="podcast-episode mod">
        <h2>{{ episode.title }}</h2>
        <div class="meta">
            <span class="date">
                {{ episode.published|date:"F j, Y" }}
            </span>
            {% for enclosure in episode.enclosure_set.all %}
                {% if enclosure.mime == 'mp3' %}
                    <span class="sep">
                        <a href="{{ enclosure.url }}" class="tiptip download"
                            title="{% trans "to download right click and select 'save link as'" %}">
                            {% trans "Download / Stream" %}
                        </a>
                    </span>
                {% endif %}
            {% endfor %}
        </div>
        <div class="episode-details">
            <a href="{{ episode.img_episode_lg.url }}" rel="facebox" class="episode-thumb">
                <img src="{{ episode.img_episode_sm.url }}"
                    alt="{{ episode.img_episode_sm.name }}"
                    width="{{ episode.img_episode_sm.width }}"
                    height="{{ episode.img_episode_sm.height }}"
                    class="left" />
            </a>
            <h6>{{ episode.subtitle }}</h6>

            {% if episode.description %}
                <div class="episode-description">{{ episode.description }}</div>
            {% endif %}

            {% if episode.tracklist %}
                <div class="tracklist">
                    <h3>{% trans "Tracklist" %}</h3>
                    {{ episode.tracklist|linenumbers|linebreaksbr }}
                </div>
            {% endif %}
        </div>
    </div>
{% else %}
    <h2 class="coming-soon">{% trans "Coming soon..." %}</h2>
{% endif %}