{% load i18n %}

<div class="podcast-show-list">
    <h2>{% trans "Podcast Shows" %}</h2>

    {% for show in show_list %}
        <div class="podcast-show">
            <h3><a href="{{ show.get_absolute_url }}">{{ show.title }}</a></h3>
            <a href="{{ show.img_show_lg.url }}" rel="facebox">
                <img src="{{ show.img_show_sm.url }}" alt="{{ show.title }}" class="left" />
            </a>
            <h4>{{ show.subtitle }}</h4>
            <p><a href="{{ show.link }}">{{ show.link }}</a></p>
            <div class="show-description">{{ show.description }}</div>
        </div>
    {% empty %}
        <h2>{% trans "There are no shows." %}</h2>
    {% endfor %}
</div>
