{% extends "podcasting/podcasting_base.html" %}

{% load i18n %}

{% block head_title %}{% trans "Current Edition" %}{% endblock %}

{% block body_class %}page-current-episode{% endblock %}

{% block content %}
    {% include "podcasting/_current_episode.html" %}
    {% block current_episode_extra %}{% endblock %}
{% endblock %}
